﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Microsoft.SharePoint.Client;
using Microsoft.SharePoint.Client.WorkflowServices;
using System.Security;
using System.Configuration;

namespace PiaSys.Workflow.FullDemo.LOBProtocol
{
    class Program
    {
        private static String username;
        private static SecureString password;

        static void Main(string[] args)
        {
            ConsoleColor defaultForeground = Console.ForegroundColor;

            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("Enter your user name (ENTER for default value: paolo@piasysdev.onmicrosoft.com):");
            Console.ForegroundColor = defaultForeground;
            username = Console.ReadLine();

            if (String.IsNullOrEmpty(username)) username = "paolo@piasysdev.onmicrosoft.com";

            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("Enter your password:");
            Console.ForegroundColor = defaultForeground;
            password = GetPasswordFromConsoleInput();

            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("Enter target file name:");
            Console.ForegroundColor = defaultForeground;
            String targetFileName = Console.ReadLine();

            Console.ForegroundColor = ConsoleColor.Green;
            Console.WriteLine("Enter the protocol number:");
            Console.ForegroundColor = defaultForeground;
            String protocolNumber = Console.ReadLine();

            using (var context = new ClientContext(getAppWebUrl("PiaSys.Workflow.FullDemo")))
            {
                context.Credentials = new SharePointOnlineCredentials(username, password);

                List targetLibrary = context.Web.Lists.GetByTitle("Contracts");
                context.Load(targetLibrary, lib => lib.Id, lib => lib.RootFolder);
                context.ExecuteQuery();

                String targetFileUrl = String.Format("{0}/{1}", targetLibrary.RootFolder.ServerRelativeUrl, targetFileName);
                File targetFile = context.Web.GetFileByServerRelativeUrl(targetFileUrl);
                ListItem targetItem = targetFile.ListItemAllFields;

                context.Load(targetItem);
                context.ExecuteQuery();

                WorkflowServicesManager wfManager = new WorkflowServicesManager(context, context.Web);
                WorkflowInstanceService wfInstanceService = wfManager.GetWorkflowInstanceService();
                WorkflowSubscriptionService wfSubscriptionService = wfManager.GetWorkflowSubscriptionService();

                context.Load(wfInstanceService);
                context.Load(wfSubscriptionService);

                context.ExecuteQuery();

                WorkflowInstanceCollection wfInstances = wfInstanceService.EnumerateInstancesForListItem(targetLibrary.Id, targetItem.Id);
                context.Load(wfInstances);
                context.ExecuteQuery();

                foreach (WorkflowInstance wfInstance in wfInstances)
                {
                    // Not completed
                    if (wfInstance.UserStatus == "Waiting for Protocol Number")
                    {
                        WorkflowSubscription wfSubscription = wfSubscriptionService.GetSubscription(wfInstance.WorkflowSubscriptionId);
                        context.Load(wfSubscription);

                        context.ExecuteQuery();

                        if (wfSubscription.Name == "ContractApprovalWorkflow - WorkflowStart")
                        {
                            wfInstanceService.PublishCustomEvent(wfInstance, "ProtocolNumberIssued", protocolNumber);
                            context.ExecuteQuery();
                        }
                    }
                }

                Console.ReadLine();
            }
        }

        private static SecureString GetPasswordFromConsoleInput()
        {
            ConsoleKeyInfo info;

            //Get the user's password as a SecureString
            SecureString securePassword = new SecureString();
            do
            {
                info = Console.ReadKey(true);
                if (info.Key != ConsoleKey.Enter)
                {
                    securePassword.AppendChar(info.KeyChar);
                }
            }
            while (info.Key != ConsoleKey.Enter);
            return securePassword;
        }

        private static String getAppWebUrl(String appName)
        {
            String appWebUrl = null;

            using (var context = new ClientContext(ConfigurationManager.AppSettings["TargetSiteUrl"]))
            {
                context.Credentials = new SharePointOnlineCredentials(username, password);

                WebCollection webs = context.Web.Webs;
                context.Load(webs);

                context.ExecuteQuery();

                foreach (Web web in webs)
                {
                    if (web.WebTemplate == "APP" && web.Title == appName)
                    {
                        appWebUrl = web.Url;
                    }
                }

                return (appWebUrl);
            }
        }
    }
}
